/*------------------------------------------------------------------------------
 * MediaTek Inc. (C) 2018. All rights reserved.
 *
 * Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 *----------------------------------------------------------------------------*/

#include <mali_kbase.h>
#include "mali_kbase_platform_mstar.h"
#include "mali_kbase_platform_mstar_m7642_reg.h"

#include <linux/delay.h>
#include "mstar/mstar_chip.h"

#if MTK_DTV_GDEC
#include <mtk_dtv_mfdec_v1.h>
#endif

void setDisableRrorWror(void)
{
	//disable rror
	SET_REG(rror, ror_inorder, 0x1);

	//disable wror
	SET_REG(wror, ror_inorder, 0x1);

	return;
}

/* Platform functions */
void init_registers(void)
{
#define MIU_BASE_TO_MB(base)        ((base) >> 20)                   /* in MB */
#define DRAM_SIZE_REG_TO_MB(reg)    ((reg > 0) ? (1 << (reg)) : 0)   /* in MB */

    /* set GPU clock */
    SET_REG(gpu_pll, ctrl0, 0x0);
    SET_REG(gpu_pll, ctrl1, (MALI_MAX_FREQ/GPU_CLOCK_STEP));
    udelay(10);

    /* default is enable, disable for zebu test */
    //setDisableRrorWror();

    /* set MIU addr */
    SET_REG(gpu, miu0_start, MIU_BASE_TO_MB(MSTAR_MIU0_BUS_BASE));
    SET_REG(gpu, miu0_end, MIU_BASE_TO_MB(MSTAR_MIU0_BUS_BASE) + DRAM_SIZE_REG_TO_MB(GET_REG(miu0, dram_size)));
#if defined(MSTAR_MIU1_BUS_BASE) && defined(ARM_MIU1_BASE_ADDR)
    SET_REG(gpu, miu1_start, MIU_BASE_TO_MB(MSTAR_MIU1_BUS_BASE));
    SET_REG(gpu, miu1_end, MIU_BASE_TO_MB(MSTAR_MIU1_BUS_BASE) + DRAM_SIZE_REG_TO_MB(GET_REG(miu1, dram_size)));
#else
    SET_REG(gpu, miu1_start, 0xffff);
    SET_REG(gpu, miu1_end, 0xffff);
#endif

    /* enable RIU */
#ifdef MSTAR_RIU_ENABLED
    SET_REG(gpu, riu, 1);
#endif

    /* set read request length 0 to 64 */
    SET_REG(gpu, rreq_rd_len0, 0x4);
    /* set read request length 1 to 64 */
    SET_REG(gpu, rreq_rd_len1, 0x4);
    /* set mfdec read request length to 64 */
    SET_REG(gpu, rreq_mfdec_len, 0x0);

    /* set gpu_hw_strobe for MIU 8x mode ([8]: strobe_inv = 0, [9]: strobe_en = 1) */
    if (GET_REG(miu0, data_ratio_4x_8x) == 0x3)
    {
        SET_REG(clkgen1, gpu_hw_strobe, 0x2);
    }

    /* enable dynamic clock gating for gpu bridge */
    SET_REG(gpu, dynamic_clk_gating_en, 1);

    /* enable GPU clock */
    SET_REG(clkgen1, ckg_gpu, 0);
    udelay(1);

    SET_REG(gpu, soft_reset, 0);    /* disable MALI soft reset */
    SET_REG(gpu, soft_reset, 1);    /* enable MALI soft reset */
    SET_REG(gpu, soft_reset, 0);    /* disable MALI soft reset */
    udelay(1);

    /* NOTICE!!!!! WROR and RROR trigger must be set after bridge register is init or reset!!!! */
    /* trigger WROR hw arb update */
    SET_REG(wror, miu_hw_arb_update, 0x0);
    SET_REG(wror, miu_hw_arb_update, 0x1);
    /* trigger RROR hw arb update */
    SET_REG(rror, miu_hw_arb_update, 0x0);
    SET_REG(rror, miu_hw_arb_update, 0x1);
}

void power_on(void)
{
    SET_REG(iommu_local_gpu, mpu_enable_gpu_local, 0); /* disable mpu for gpu local */
    SET_REG(clkgen1, ckg_gpu, 0);   /* enable GPU clock */
    SET_REG(gpu, sram_sd_en, 0);    /* power on SRAM */

    /* PULL DOWN GPU RESET */
    SET_REG(gpu, soft_reset, 0);
    udelay(1);

    /* NOTICE!!!!! WROR and RROR trigger must be set after bridge register is init or reset!!!! */
    /* trigger WROR hw arb update */
    SET_REG(wror, miu_hw_arb_update, 0x0);
    SET_REG(wror, miu_hw_arb_update, 0x1);
    /* trigger RROR hw arb update */
    SET_REG(rror, miu_hw_arb_update, 0x0);
    SET_REG(rror, miu_hw_arb_update, 0x1);

#if MTK_DTV_GDEC
    init_mfdec();
#endif
}

void power_off(void)
{
    SET_REG(gpu, soft_reset, 1);    /* reset GPU */
    SET_REG(gpu, sram_sd_en, 1);    /* power off SRAM */
    SET_REG(clkgen1, ckg_gpu, 1);   /* disable GPU clock */
    udelay(1);
}

void soft_reset(void)
{
    /* reset GPU hardcore & external GPU bridge circuit */
    SET_REG(gpu, soft_reset, 1);
    udelay(1);
    SET_REG(gpu, soft_reset, 0);

    /* GPU secure sw reset will clear one-way register (0x5b[0], reg_gpu_dram_mask_en)
     * and let GPU use normal AID, which can only R/W normal MIU range.
     */
    SET_REG(gpu, secure_sw_reset, 1);
    udelay(1);
    SET_REG(gpu, secure_sw_reset, 0);
}

#ifdef MSTAR_DISABLE_SHADER_CORES
int get_num_disabled_cores(void)
{
    return 0;
}
#endif

void get_dram_length(u32* miu0_length, u32* miu1_length, u32* miu2_length)
{
#define DRAM_SIZE_REG_TO_MB(reg)    ((reg > 0) ? (1 << (reg)) : 0)      /* in MB */

    *miu0_length = DRAM_SIZE_REG_TO_MB(GET_REG(miu0, dram_size));
#if defined(MSTAR_MIU1_BUS_BASE) && defined(ARM_MIU1_BASE_ADDR)
    *miu1_length = DRAM_SIZE_REG_TO_MB(GET_REG(miu1, dram_size));
#else
    *miu1_length = 0;
#endif
    *miu2_length = 0;

    return;
}
