/*------------------------------------------------------------------------------
 * MediaTek Inc. (C) 2020. All rights reserved.
 *
 * Copyright Statement:
 *
 * This software/firmware and related documentation ("MediaTek Software") are
 * protected under relevant copyright laws. The information contained herein is
 * confidential and proprietary to MediaTek Inc. and/or its licensors. Without
 * the prior written permission of MediaTek inc. and/or its licensors, any
 * reproduction, modification, use or disclosure of MediaTek Software, and
 * information contained herein, in whole or in part, shall be strictly
 * prohibited.
 *
 * BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
 * THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("MEDIATEK SOFTWARE")
 * RECEIVED FROM MEDIATEK AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
 * ON AN "AS-IS" BASIS ONLY. MEDIATEK EXPRESSLY DISCLAIMS ANY AND ALL
 * WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NONINFRINGEMENT. NEITHER DOES MEDIATEK PROVIDE ANY WARRANTY WHATSOEVER WITH
 * RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
 * INCORPORATED IN, OR SUPPLIED WITH THE MEDIATEK SOFTWARE, AND RECEIVER AGREES
 * TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
 * RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
 * OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN MEDIATEK
 * SOFTWARE. MEDIATEK SHALL ALSO NOT BE RESPONSIBLE FOR ANY MEDIATEK SOFTWARE
 * RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
 * STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND MEDIATEK'S
 * ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE MEDIATEK SOFTWARE
 * RELEASED HEREUNDER WILL BE, AT MEDIATEK'S OPTION, TO REVISE OR REPLACE THE
 * MEDIATEK SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
 * CHARGE PAID BY RECEIVER TO MEDIATEK FOR SUCH MEDIATEK SOFTWARE AT ISSUE.
 *
 * The following software/firmware and/or related documentation ("MediaTek
 * Software") have been modified by MediaTek Inc. All revisions are subject to
 * any receiver's applicable license agreements with MediaTek Inc.
 *----------------------------------------------------------------------------*/

#include <mali_kbase.h>
#include <mtk_dtv_gpu_bridge.h>
#include <linux/delay.h>

#ifndef _KBASE_PLATFORM_MTK_DTV_MFDEC_V3_H_
#define _KBASE_PLATFORM_MTK_DTV_MFDEC_V3_H_

#define MAX_MFDEC_SLOT_NUM 16

#define FB_TYPE_L_BASE   0
#define FB_TYPE_C_BASE   1
#define FB_TYPE_PITCH    2
#define FB_TYPE_PROP     3

bridge_device mfdec_bridge;

void init_mfdec_reg(void)
{
    SET_REG(disp_mfdec0, mfdec_detile_rst, 1);
    udelay(1);
    SET_REG(disp_mfdec0, mfdec_detile_rst, 0);

    /* set mfdec read request length to 64 */
    SET_REG(gpu, mfdec_rd_len, 0x0);
    /* enable divider, no need to set pitch as power of 2 */
    SET_REG(gpu, mfdec_divider_en, 1);
    udelay(1);

    /* enable mfdec */
    SET_REG(gpu, mfdec_off, 0);
    udelay(1);
}

void init_mfdec(void)
{
    init_bridge_device(&mfdec_bridge);
    init_mfdec_reg();
}

int set_mfdec_bridge(u8 id, u64 base, u32 height, u32 pitch, u8 is_tiled_yuv)
{
    u32 size_y, size_uv;
    u64 base_uv;
    u16 valid_id = 0x0;

    if (id == 0xFF)
    {
        id = acquire_slot(&mfdec_bridge, MAX_MFDEC_SLOT_NUM);
        if (id == 0xFF)
            return -1;
        else if (id == 0x0)
        {
            /* Re-init mfdec reg when first slot is occupied */
            init_mfdec_reg();
        }
        mfdec_used_id += (1 << id);
    }

    SET_REG(gpu, mfdec_adr_idx, (id & 0xF));

    /* Set frame pitch (roundup to power of 2) */
    SET_REG(gpu, video_pitch, pitch);

    /* Bifrost MMU table is bus address */
    size_y = ((pitch * height + 0xFFF) >> 12) << 12;
    base_uv = base + size_y;
    /* size of YUV_420SP(NV12) format */
    size_uv = ((2 * (pitch * height / 4) + 0xFFF) >> 12) << 12;

    /* Set start & end addr of luma & chroma (4K byte unit, bus address) */
    SET_REG(gpu, mfdec_y_start_low,   ((base >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_y_start_high,  ((base >> 28) & 0xFFFF));
    SET_REG(gpu, mfdec_y_end_low,     (((base + size_y) >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_y_end_high,    (((base + size_y) >> 28) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_start_low,  ((base_uv >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_start_high, ((base_uv >> 28) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_end_low,    (((base_uv + size_uv) >> 12) & 0xFFFF));
    SET_REG(gpu, mfdec_uv_end_high,   (((base_uv + size_uv) >> 28) & 0xFFFF));

    /* Update & clean */
    SET_REG(gpu, mfdec_adr_update,    0x1);
    SET_REG(gpu, mfdec_adr_update,    0x0);

    /* Set valid vector, do not clear any valid bit while GPU is working */
    valid_id = GET_REG(gpu, mfdec_valid);
    valid_id |= (0x1 << id);
    SET_REG_NO_CLEAR(gpu, mfdec_valid,         (valid_id & 0xFFFF));

    return id;
}

int set_mfdec_detile(u8 id, u64 video_ybase, u64 video_uvbase, u32 pitch, u32 tile_mode, u32 flags)
{
    if (id == 0xFF)
        return -1;

    /* Set frame buffer index */
    SET_REG(disp_mfdec0, prg_fb_idx, id);

    /* Set frame buffer luma base */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_L_BASE);
    SET_REG(disp_mfdec0, fb_para_data_low, video_ybase);
    SET_REG(disp_mfdec0, fb_para_data_high, (video_ybase >> 16));
    udelay(1);

    /* Set frame buffer chroma base */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_C_BASE);
    SET_REG(disp_mfdec0, fb_para_data_low, video_uvbase);
    SET_REG(disp_mfdec0, fb_para_data_high, (video_uvbase >> 16));
    udelay(1);

    /* Set frame buffer pitch */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PITCH);
    SET_REG(disp_mfdec0, fb_para_data_low, pitch);
    SET_REG(disp_mfdec0, fb_para_data_high, 0x0);
    udelay(1);

    /* Set frame buffer property */
    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PROP);
    {
        u8 offset = 0x0;
        u32 property = 0x0;

        /* For IOMMU */
        offset = (((video_uvbase) >> 32) << 2) & (0xf);
        property += offset << 8;
        offset = (((video_ybase) >> 32) << 2) & (0xf);
        property += offset << 4;
        property += tile_mode;
        SET_REG(disp_mfdec0, fb_para_data_low, property);

        property = 0x0;
        if (flags != 0)
        {
            property = property | (flags >> 16);
        }
        SET_REG(disp_mfdec0, fb_para_data_high, property);
    }

    return 0;
}

int set_mfdec_decode(u8 id, u64 video_ybase, u64 video_uvbase, u64 bld_base, u32 bld_pitch, u32 codec_type, u32 flags)
{
    pr_alert("Does not support mfdec decode! Detile only.\n");
    return -1;
}

int reset_mfdec_slot(u8 id)
{
    if (id >= MAX_MFDEC_SLOT_NUM)
        return -1;

    /* Set invalid vector */
    CLEAR_REG_VAL(gpu, mfdec_valid, (0x1 << id));

    release_slot(&mfdec_bridge, id);
    mfdec_used_id -= (1 << id);

    return 0;
}

void print_debug_status(void)
{
    printk(KERN_DEBUG " [Mali-DEBUG] mfdec_off      = 0x%x\n", GET_REG(gpu, mfdec_off));
    printk(KERN_DEBUG " [Mali-DEBUG] mfdec_used_id  = 0x%x\n", mfdec_used_id);

    /* Check the MFDEC bank status */
    printk(KERN_DEBUG " [Mali-DEBUG][MFDEC] INDEX           = 0x%x\n", GET_REG(disp_mfdec0, prg_fb_idx));

    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_L_BASE);
    printk(KERN_DEBUG " [Mali-DEBUG][MFDEC] Y_BASE  = 0x%04x %04x\n", GET_REG(disp_mfdec0, fb_para_data_high), GET_REG(disp_mfdec0, fb_para_data_low));

    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_C_BASE);
    printk(KERN_DEBUG " [Mali-DEBUG][MFDEC] C_BASE  = 0x%04x %04x\n", GET_REG(disp_mfdec0, fb_para_data_high), GET_REG(disp_mfdec0, fb_para_data_low));

    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PITCH);
    printk(KERN_DEBUG " [Mali-DEBUG][MFDEC] PITCH   = 0x%x %x\n", GET_REG(disp_mfdec0, fb_para_data_high), GET_REG(disp_mfdec0, fb_para_data_low));

    SET_REG(disp_mfdec0, prg_fb_type, FB_TYPE_PROP);
    printk(KERN_DEBUG " [Mali-DEBUG][MFDEC] PROP    = 0x%x %x\n", GET_REG(disp_mfdec0, fb_para_data_high), GET_REG(disp_mfdec0, fb_para_data_low));


    printk(KERN_DEBUG " [Mali-DEBUG][BRIDGE] ID       = 0x%x\n", GET_REG(gpu, mfdec_adr_idx));

    printk(KERN_DEBUG " [Mali-DEBUG][BRIDGE] PITCH    = 0x%x\n", GET_REG(gpu, video_pitch));

    printk(KERN_DEBUG " [Mali-DEBUG][BRIDGE] Y_START  = 0x%04x %04x\n", GET_REG(gpu, mfdec_y_start_high), GET_REG(gpu, mfdec_y_start_low));
    printk(KERN_DEBUG " [Mali-DEBUG][BRIDGE] Y_END    = 0x%04x %04x\n", GET_REG(gpu, mfdec_y_end_high), GET_REG(gpu, mfdec_y_end_low));
    printk(KERN_DEBUG " [Mali-DEBUG][BRIDGE] UV_START = 0x%04x %04x\n", GET_REG(gpu, mfdec_uv_start_high), GET_REG(gpu, mfdec_uv_start_low));
    printk(KERN_DEBUG " [Mali-DEBUG][BRIDGE] UV_END   = 0x%04x %04x\n", GET_REG(gpu, mfdec_uv_end_high), GET_REG(gpu, mfdec_uv_end_low));

    printk(KERN_DEBUG " [Mali-DEBUG][BRIDGE] VALID    = 0x%x\n", GET_REG(gpu, mfdec_valid));
}
#endif /* _KBASE_PLATFORM_MTK_DTV_MFDEC_V3_H_ */
